@@ -22,7 +22,7 @@
 
 static GtkWidget *cache_description(appdata_t *appdata, cache_t *cache) {
   return html_view(appdata, cache->long_description, 
-		   cache->long_is_html, TRUE, cache, NULL);
+	   cache->long_is_html?HTML_HTML:HTML_PLAIN_TEXT, TRUE, cache, NULL);
 }
 
 #ifndef USE_MAEMO  // maemos touchscreen doesn't support tooltips
@@ -155,7 +155,6 @@ static GtkWidget *cache_overview(cache_context_t *context) {
     gtk_box_pack_start_defaults(GTK_BOX(ivbox), GTK_LABEL_SMALL(_("by")));
 
     static const char *owner_type = "profile/";
-    /* todo: small!!! */
     gtk_box_pack_start_defaults(GTK_BOX(ivbox), 
 		link_button_by_id(appdata, cache->owner->name, 
 				  owner_type, cache->owner->id));
@@ -227,7 +226,7 @@ static GtkWidget *cache_overview(cache_context_t *context) {
   if(cache->short_description)
     gtk_box_pack_start_defaults(GTK_BOX(vbox), 
 	html_view(appdata, cache->short_description, 
-		  cache->short_is_html, TRUE, cache, NULL));
+	  cache->short_is_html?HTML_HTML:HTML_PLAIN_TEXT, TRUE, cache, NULL));
 
   return vbox;
 }
@@ -273,14 +272,15 @@ static void on_decrypt(GtkWidget *widget, gpointer data) {
 static GtkWidget *cache_hint(appdata_t *appdata, cache_t *cache) {
   /* encrypting/decrypting html is nothing we want to do */
   if(cache->hint_is_html) 
-    return html_view(appdata, cache->hint, TRUE, TRUE, NULL, NULL);
+    return html_view(appdata, cache->hint, HTML_HTML, TRUE, NULL, NULL);
 
   /* we can now be sure that we are talking about pain text */
   GtkWidget *vbox = gtk_vbox_new(FALSE, 0);
 
   char *hint = strdup(cache->hint);
   rot13(hint);
-  GtkWidget *view = html_view(appdata, hint, FALSE, TRUE, NULL, NULL);
+  GtkWidget *view = 
+    html_view(appdata, hint, HTML_PLAIN_TEXT, TRUE, NULL, NULL);
   gtk_box_pack_start_defaults(GTK_BOX(vbox), view);
   free(hint);
 
@@ -522,9 +522,17 @@ static GtkWidget *cache_logs(appdata_t *appdata, cache_context_t *context, log_t
   while(log) {
     gtk_table_attach_defaults(GTK_TABLE(table), gtk_hseparator_new(), 
 			      0, 3, cnt+0, cnt+1);
+#if 0
+    static const char *log_type = "seek/log.aspx";
+    GtkWidget *log_but = 
+      link_icon_button_by_id(appdata, icon_get_widget(ICON_LOG, log->type), 
+			     log_type, log->id);
+    gtk_table_attach(GTK_TABLE(table), log_but, 
+		     0, 1, cnt+1, cnt+2, FALSE, FALSE, 0, 0);
+#else
     gtk_table_attach_defaults(GTK_TABLE(table), 
-			      icon_get_widget(ICON_LOG, log->type), 
-			      0, 1, cnt+1, cnt+2);
+	      icon_get_widget(ICON_LOG, log->type), 0, 1, cnt+1, cnt+2);
+#endif
 
     char date_str[32];
     if(log->day && log->month && log->year) {
@@ -549,10 +557,11 @@ static GtkWidget *cache_logs(appdata_t *appdata, cache_context_t *context, log_t
 
     if(log->text) {
       gtk_table_attach_defaults(GTK_TABLE(table), 
-		html_view(appdata, log->text, is_html, FALSE, NULL, NULL),
-		0, 3, cnt+3, cnt+4);
+	html_view(appdata, log->text, 
+		  is_html?HTML_HTML:HTML_CUSTOM_MARKUP, FALSE, NULL, NULL),
+				0, 3, cnt+3, cnt+4);
     }
-
+    
     log = log->next;
     cnt+=4;
   }
@@ -597,13 +606,13 @@ static void on_notebook_page_change(GtkNotebook     *notebook,
 
   /* this is a workaround, around some bug in the gtktextwidget or so ... */
   /* i tried to get info on this and everybody agreed that this is a bug */
-  /* in gtk but noone had a read fix. so i came up with this. */
+  /* in gtk but noone had a fix ready. so i came up with this. */
   /* seems to work ... */
   if(strcasecmp(name, _("Logs")) == 0) {
     gtk_widget_queue_resize(w);
-  }
-
-  if(strcasecmp(name, _("Goto")) == 0) {
+  } else if(strcasecmp(name, _("TBs")) == 0) {
+    gtk_widget_queue_resize(w);
+  } else if(strcasecmp(name, _("Goto")) == 0) {
 #ifdef USE_MAEMO  
     context->handler_id = gtk_timeout_add(1000, screensaver_update, 
 					  context->appdata);
