@@ -78,11 +78,11 @@ static store_t store[] = {
   { "cachelist_items",  STORE_INT,    OFFSET(cachelist_items) },
   { "compass_damping",  STORE_INT,    OFFSET(compass_damping) },
   { "cachelist_hide_found", STORE_BOOL, OFFSET(cachelist_hide_found) },
+  { "cachelist_update", STORE_BOOL,   OFFSET(cachelist_update) },
 #ifdef USE_MAEMO
   { "mmpoi_dontlaunch", STORE_BOOL,   OFFSET(mmpoi_dontlaunch) },
   { "cachelist_dss",    STORE_BOOL,   OFFSET(cachelist_disable_screensaver) },
   { "goto_dss",         STORE_BOOL,   OFFSET(goto_disable_screensaver) },
-  { "cachelist_update", STORE_BOOL,   OFFSET(cachelist_update) },
 #endif
 #ifdef ENABLE_OSM_GPS_MAP
   { "map_lat",          STORE_FLOAT,  OFFSET(map.pos.lat) },
@@ -324,6 +324,10 @@ void gconf_load_state(appdata_t *appdata) {
     appdata->search = SEARCH_NAME | SEARCH_ID;
 
   if(!appdata->image_path) {
+#ifdef USE_MAEMO
+    /* update cachelist by default */
+    appdata->cachelist_update = TRUE;
+#endif
 
     /* use gps by default */
     appdata->use_gps = TRUE;
