@@ -642,3 +642,13 @@ gps_sat_t *gps_get_sats(appdata_t *appdata) {
 
 
 #endif
+
+void *gps_register_callback(appdata_t *appdata, gps_cb cb, gpointer data) {
+  printf("register gps callback\n");
+  return NULL;
+}
+
+
+void gps_unregister_callback(appdata_t *appdata, void *cb) {
+  printf("unregister gps callback\n");
+}
