@@ -119,8 +119,8 @@ void gpx_free_all(gpx_t *gpx) {
 }
 
 static const char *cache_type_str[] = { "<Unknown>",
-  "Traditional Cache|Traditional|Geocache", "Multi-cache|Multi", 
-  "Unknown Cache|Other",
+  "Traditional Cache|Traditional|Geocache", "Multi-cache|Multi|Multicache", 
+  "Unknown Cache|Other|Unknown",
   "Virtual Cache|Virtual", "Webcam Cache|Webcam", "Event Cache|Event|Geocoins:",
   "Letterbox Hybrid|Letterbox", "Earthcache", "Wherigo Cache",
   "Mega-Event Cache", "Cache In Trash Out Event",
@@ -273,6 +273,7 @@ void gpx_display_log(log_t *log) {
   printf("    date:     %d.%d.%d\n", log->day, log->month, log->year);
   printf("    type:     %s\n", log_type_str[log->type+1]);
   printf("    finder:   %s\n", log->finder->name);
+  printf("    id:       #%u\n", log->id);
   //  printf("    text:     %s\n", log->text);
 }
 
@@ -440,8 +441,11 @@ static log_t *process_gpx_wpt_gc_logs_log(xmlTextReaderPtr reader) {
     return NULL;
 
   /* create a new log entry */
-  log_t *log = malloc(sizeof(log_t));
-  memset(log, 0, sizeof(log_t));
+  log_t *log = g_new0(log_t, 1);
+
+  char *prop = (char*)xmlTextReaderGetAttribute(reader, BAD_CAST "id");
+  if(prop)
+    log->id = atoi(prop);
 
   /* process all sub-nodes */
   int depth = xmlTextReaderDepth(reader);
