@@ -269,5 +269,7 @@ extern gboolean on_window_key_press(GtkWidget *widget,
 #ifdef USE_STACKABLE_WINDOW
 HildonAppMenu *menu_create(appdata_t *appdata, int mode);
 #endif
+void cachelist_goto_cache(appdata_t *appdata, cache_t *cache);
+void gpxlist_goto_cachelist(appdata_t *appdata, gpx_t *gpx);
 
 #endif // GPXVIEW_H
