@@ -362,11 +362,11 @@ static const char *html_start = "<html><head>"
 static const char *html_end = "</body></html>";
 
 GtkWidget *html_view(appdata_t *appdata, char *text, 
-		     gboolean is_html, gboolean scrollwin,
+		     html_mode_t mode, gboolean scrollwin,
 		     cache_t *cache, char *anchor) {
   GtkWidget *view;
 
-  if(is_html) {
+  if(mode == HTML_HTML) {
     http_context_t *context = g_new0(http_context_t, 1);
     context->appdata = appdata;
     context->cache = cache;
@@ -453,7 +453,7 @@ GtkWidget *html_view(appdata_t *appdata, char *text,
 #endif
 #else
 #ifndef PANNABLE_HTML
-  if(is_html) { 
+  if(mode == HTML_HTML) { 
       GtkWidget *scrolled_window = gtk_scrolled_window_new(NULL, NULL);
       gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW (scrolled_window), 
 		     GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
