@@ -80,6 +80,7 @@ static const char *get_proxy_uri(appdata_t *appdata) {
 
 static void
 cb_map_gps(osd_button_t but, map_context_t *context) {
+
   if(but == OSD_GPS) {
     pos_t *refpos = get_pos(context->appdata);
     if(refpos && !isnan(refpos->lat) && !isnan(refpos->lon)) {
@@ -108,14 +109,18 @@ static int dist2pixel(map_context_t *context, float km, float lat) {
 
 static gboolean map_gps_update(gpointer data) {
   map_context_t *context = (map_context_t*)data;
+  static gboolean goto_is_enabled = FALSE;
 
   /* get reference position ... */
   pos_t *refpos = get_pos(context->appdata);
   gboolean ok = (refpos!= NULL) && !isnan(refpos->lat) && !isnan(refpos->lon);
 
   /* ... and enable "goto" button if it's valid */
-  osm_gps_map_osd_enable_gps (OSM_GPS_MAP(context->widget), 
-		 OSM_GPS_MAP_OSD_CALLBACK(ok?cb_map_gps:NULL), context);
+  if(ok != goto_is_enabled) {
+    osm_gps_map_osd_enable_gps (OSM_GPS_MAP(context->widget), 
+		OSM_GPS_MAP_OSD_CALLBACK(ok?cb_map_gps:NULL), context);
+    goto_is_enabled = ok;
+  }
 
   if(ok) {
     float heading = NAN;
@@ -130,6 +135,8 @@ static gboolean map_gps_update(gpointer data) {
 	radius = dist2pixel(context, eph/1000, refpos->lat);
     }
 
+    /* TODO: in order to save energy: only draw if state actually changed */
+
     g_object_set(context->widget, "gps-track-highlight-radius", radius, NULL);
     osm_gps_map_draw_gps(OSM_GPS_MAP(context->widget), 
 			 refpos->lat, refpos->lon, heading);
@@ -139,45 +146,9 @@ static gboolean map_gps_update(gpointer data) {
   return TRUE;
 }
 
-static gboolean on_map_configure(GtkWidget *widget,
-				 GdkEventConfigure *event,
-				 map_context_t *context) {
-
-  if(!context->map_complete) {
-
-    /* set default values if they are invalid */
-    if(!context->appdata->map.zoom || 
-       isnan(context->appdata->map.pos.lat) ||
-       isnan(context->appdata->map.pos.lon)) {
-      printf("no valid map position found\n");
-      
-      pos_t *refpos = get_pos(context->appdata);
-      if(refpos && !isnan(refpos->lat) && !isnan(refpos->lon)) {
-	/* use gps position if present */
-	context->appdata->map.pos = *refpos;
-	context->appdata->map.zoom = GPS_DEFAULT_ZOOM;
-      } else {
-	/* use world map otherwise */
-	context->appdata->map.pos.lat = 0.0;
-	context->appdata->map.pos.lon = 0.0;
-	context->appdata->map.zoom = 1;
-      }
-    }
-    
-    /* jump to initial position */
-    osm_gps_map_set_mapcenter(OSM_GPS_MAP(context->widget), 
-			      context->appdata->map.pos.lat, 
-			      context->appdata->map.pos.lon, 
-			      context->appdata->map.zoom);
-    context->map_complete = TRUE;
-  }
-
-  return FALSE;
-}
-
 static void map_draw_cache(GtkWidget *map, cache_t *cache) {
   GdkPixbuf *icon = icon_get(ICON_CACHE_TYPE, cache->type);
-    
+
   /* check if there's also an overwritten coordinate */
   if(cache->notes && cache->notes->override) {
     GdkPixbuf *over = icon_get(ICON_MISC, 1);
@@ -187,10 +158,11 @@ static void map_draw_cache(GtkWidget *map, cache_t *cache) {
 
     osm_gps_map_add_image(OSM_GPS_MAP(map),
 	  cache->notes->pos.lat, cache->notes->pos.lon, over);
-  } else
-    osm_gps_map_add_image(OSM_GPS_MAP(map), 
-			  cache->pos.lat, cache->pos.lon, icon);
-
+  } else {
+    if(!isnan(cache->pos.lat) && !isnan(cache->pos.lon))
+      osm_gps_map_add_image(OSM_GPS_MAP(map), 
+			    cache->pos.lat, cache->pos.lon, icon);
+  }
 }
 
 static void map_draw_gpx(appdata_t *appdata, GtkWidget *map, gpx_t *gpx) {
@@ -206,9 +178,148 @@ static void map_draw_gpx(appdata_t *appdata, GtkWidget *map, gpx_t *gpx) {
   }
 }
 
+/* draw geocaches and set window title */
+static void map_setup(map_context_t *context) {
+  char *name = NULL;
+
+  if(!context->appdata->cur_gpx && !context->appdata->cur_cache) {
+    if(context->state != MAP_ALL) {
+      printf("map_setup(ALL)\n");
+
+#ifdef OSD_NAV
+      /* no navigation in this mode */
+      osm_gps_map_osd_clear_nav (OSM_GPS_MAP(context->widget));
+#endif
+
+      /* clear all existing ccahe images */
+      osm_gps_map_clear_images (OSM_GPS_MAP(context->widget));
+
+      /* draw all geocaches */
+      gpx_t *gpx = context->appdata->gpx;
+      while(gpx) {
+	map_draw_gpx(context->appdata, context->widget, gpx);
+	gpx = gpx->next;
+      }
+      name = g_strdup(_("all"));
+      context->state = MAP_ALL;
+    }
+  } else if(!context->appdata->cur_cache) {
+    if(context->state != MAP_GPX) {
+      printf("map_setup(GPX)\n");
+
+#ifdef OSD_NAV
+      /* no navigation in this mode */
+      osm_gps_map_osd_clear_nav (OSM_GPS_MAP(context->widget));
+#endif
+
+      /* clear all existing ccahe images */
+      osm_gps_map_clear_images (OSM_GPS_MAP(context->widget));
+
+      map_draw_gpx(context->appdata, context->widget, 
+		   context->appdata->cur_gpx);
+      name = g_strdup(context->appdata->cur_gpx->name);
+      context->state = MAP_GPX;
+    }
+  } else {
+    cache_t *cache = context->appdata->cur_cache;
+    
+    printf("map_setp(CACHE)\n");
+    
+    /* no balloons in this mode */
+    context->balloon = NULL;
+    osm_gps_map_osd_clear_balloon (OSM_GPS_MAP(context->widget));
+    
+    /* clear all existing ccahe images */
+    osm_gps_map_clear_images (OSM_GPS_MAP(context->widget));
+    
+    map_draw_cache(context->widget, cache);
+    name = g_strdup(cache->name);
+    context->state = MAP_CACHE;
+    
+    /* navigation in this mode! */
+    pos_t cpos = gpx_cache_pos(cache);
+
+#ifdef OSD_NAV
+    osm_gps_map_osd_draw_nav (OSM_GPS_MAP(context->widget), 
+			      context->appdata->imperial,
+			      cpos.lat, cpos.lon, cache->name);
+#else
+#warning OSD_NAV not defined!
+#endif
+  }
+
+  if(name) {
+    char *title = g_strdup_printf(_("Map - %s"), name);
+    g_free(name);
+
+    gtk_window_set_title(GTK_WINDOW(context->window), title);
+    
+    g_free(title);
+  } else
+    printf("map_setup(keep)\n");
+}
+
+static gboolean on_map_configure(GtkWidget *widget,
+				 GdkEventConfigure *event,
+				 map_context_t *context) {
+
+  /* for some reason there's a configure event with 1/1 */
+  /* on diablo. We just ignore this! */
+
+  printf("on_map_configure %d %d\n", 
+	 widget->allocation.width, 
+	 widget->allocation.height);
+
+  if(!context->map_complete && 
+     (widget->allocation.width > 100) &&
+     (widget->allocation.height > 100)) {
+
+    /* setup cache state */
+    map_setup(context);
+
+    /* set default values if they are invalid */
+    if(!context->appdata->map.zoom || 
+       isnan(context->appdata->map.pos.lat) ||
+       isnan(context->appdata->map.pos.lon)) {
+      printf("no valid map position found\n");
+      
+      pos_t *refpos = get_pos(context->appdata);
+      if(refpos && !isnan(refpos->lat) && !isnan(refpos->lon)) {
+	printf("use refpos\n");
+
+	/* use gps position if present */
+	context->appdata->map.pos = *refpos;
+	context->appdata->map.zoom = GPS_DEFAULT_ZOOM;
+      } else {
+	printf("use zero pos\n");
+
+	/* use world map otherwise */
+	context->appdata->map.pos.lat = 0.0;
+	context->appdata->map.pos.lon = 0.0;
+	context->appdata->map.zoom = 1;
+      }
+    }
+    
+    /* jump to initial position */
+    printf("osm_gps_map_set_mapcenter(%f,%f,%d)\n", 
+	   context->appdata->map.pos.lat,
+	   context->appdata->map.pos.lon,
+	   context->appdata->map.zoom);
+	   
+    osm_gps_map_set_mapcenter(OSM_GPS_MAP(context->widget), 
+			      context->appdata->map.pos.lat, 
+			      context->appdata->map.pos.lon, 
+			      context->appdata->map.zoom);
+    context->map_complete = TRUE;
+  }
+
+  return FALSE;
+}
+
 static void
 map_cachelist_nearest(cache_t *cache, pos_t *pos, 
 		      cache_t **result, float *distance) {
+
   while(cache) {
     pos_t cpos = gpx_cache_pos(cache);
 
@@ -258,10 +369,11 @@ pos_t coord2pos(coord_t coo) {
 static gboolean
 on_map_button_press_event(GtkWidget *widget, 
 			    GdkEventButton *event, map_context_t *context) {
+
   OsmGpsMap *map = OSM_GPS_MAP(context->widget);
 
   /* check if we actually clicked parts of the OSD */
-  if(osm_gps_map_osd_check(map, event->x, event->y) != OSD_NONE)
+  if(osm_gps_map_osd_check(map, event->x, event->y) != OSD_NONE) 
     return FALSE;
 
   /* got a press event without release event? eat it! */
@@ -315,121 +427,182 @@ cairo_draw_pixbuf(cairo_t *cr, GdkPixbuf *buf, gint x, gint y) {
 #endif
 
 static void 
-balloon_draw_cb(cairo_t *cr, OsmGpsMapRect_t *rect, gpointer data) {
-  cache_t *cache = (cache_t*)data;
+balloon_cb(osm_gps_map_balloon_event_t *event, gpointer data) {
+  printf("balloon event: ");
+
+  map_context_t *context = (map_context_t*)data;
+  cache_t *cache = context->balloon;
+
+  if(event->type == OSM_GPS_MAP_BALLOON_EVENT_TYPE_DRAW) {
+    printf("draw\n");
 
 #if 0
-  /* draw pink background to check clipping */
-  cairo_rectangle (cr, rect->x-20, rect->y-20, rect->w+40, rect->h+40);
-  cairo_set_source_rgba (cr, 1, 0, 0, 0.3);
-  cairo_fill_preserve (cr);
-  cairo_set_line_width (cr, 0);
-  cairo_stroke (cr);
+    /* draw pink background to check clipping */
+    cairo_rectangle (event->data.draw.cr, 
+		     event->data.draw.rect->x-20, event->data.draw.rect->y-20, 
+		     event->data.draw.rect->w+40, event->data.draw.rect->h+40);
+    cairo_set_source_rgba (event->data.draw.cr, 1, 0, 0, 0.3);
+    cairo_fill_preserve (event->data.draw.cr);
+    cairo_set_line_width (event->data.draw.cr, 0);
+    cairo_stroke (event->data.draw.cr);
 #endif
-
-  /* leave a little border top and left */
-  gint x = rect->x, y = rect->y;
-
-  /* draw the cache type icon ... */
-  GdkPixbuf *icon = icon_get(ICON_CACHE_TYPE, cache->type);
-  cairo_draw_pixbuf(cr, icon, x, y);
-
-  if(cache->notes && cache->notes->override) {
-    GdkPixbuf *over = icon_get(ICON_MISC, 1);
-    cairo_draw_pixbuf(cr, over, x, y);
-  }
-
-  /* ... and right of it the waypoint id */
-  cairo_text_extents_t extents;
-
-  if(cache->id) {
-    cairo_select_font_face (cr, "Sans",
-			    CAIRO_FONT_SLANT_NORMAL,
-			    CAIRO_FONT_WEIGHT_BOLD);
-  
+    
+    /* leave a little border top and left */
+    gint x = event->data.draw.rect->x, y = event->data.draw.rect->y;
+    
+    /* draw the cache type icon ... */
+    GdkPixbuf *icon = icon_get(ICON_CACHE_TYPE, cache->type);
+    cairo_draw_pixbuf(event->data.draw.cr, icon, x, y);
+    
+    if(cache->notes && cache->notes->override) {
+      GdkPixbuf *over = icon_get(ICON_MISC, 1);
+      cairo_draw_pixbuf(event->data.draw.cr, over, x, y);
+    }
+    
+    /* ... and right of it the waypoint id */
+    cairo_text_extents_t extents;
+    
+    if(cache->id) {
+      cairo_select_font_face (event->data.draw.cr, "Sans",
+			      CAIRO_FONT_SLANT_NORMAL,
+			      CAIRO_FONT_WEIGHT_BOLD);
+      
 #ifndef BIG_BALLOONS
-    cairo_set_font_size (cr, 20.0);
+      cairo_set_font_size (event->data.draw.cr, 20.0);
 #else
-    cairo_set_font_size (cr, 36.0);
+      cairo_set_font_size (event->data.draw.cr, 36.0);
 #endif
-
-    cairo_text_extents (cr, cache->id, &extents);
-
-    /* display id right of icon vertically centered */
-    x += gdk_pixbuf_get_width(icon) + 5;
-    y += (gdk_pixbuf_get_height(icon) + extents.height)/2;
-    cairo_move_to (cr, x, y);
-    cairo_set_source_rgba (cr, 0, 0, 0, 1);
-    cairo_show_text (cr, cache->id);
-    cairo_stroke (cr);
-
-    y += (gdk_pixbuf_get_height(icon) - extents.height)/2 + LINE_SKIP;
-  } else
-    y += gdk_pixbuf_get_height(icon);
-
-  /* return to the left border and below icon/text */
-  x = rect->x;
-
-  /* everything from here uses the same font */
-  cairo_select_font_face (cr, "Sans", CAIRO_FONT_SLANT_NORMAL,
-  			  CAIRO_FONT_WEIGHT_NORMAL);
+      
+      cairo_text_extents (event->data.draw.cr, cache->id, &extents);
+      
+      /* display id right of icon vertically centered */
+      x += gdk_pixbuf_get_width(icon) + 5;
+      y += (gdk_pixbuf_get_height(icon) + extents.height)/2;
+      cairo_move_to (event->data.draw.cr, x, y);
+      cairo_set_source_rgba (event->data.draw.cr, 0, 0, 0, 1);
+      cairo_show_text (event->data.draw.cr, cache->id);
+      cairo_stroke (event->data.draw.cr);
+      
+      y += (gdk_pixbuf_get_height(icon) - extents.height)/2 + LINE_SKIP;
+    } else
+      y += gdk_pixbuf_get_height(icon);
+    
+    /* return to the left border and below icon/text */
+    x = event->data.draw.rect->x;
+    
+    /* everything from here uses the same font */
+    cairo_select_font_face (event->data.draw.cr, "Sans", CAIRO_FONT_SLANT_NORMAL,
+			    CAIRO_FONT_WEIGHT_NORMAL);
 #ifndef BIG_BALLOONS
-  cairo_set_font_size (cr, 14.0);
+    cairo_set_font_size (event->data.draw.cr, 14.0);
 #else
-  cairo_set_font_size (cr, 22.0);
+    cairo_set_font_size (event->data.draw.cr, 22.0);
 #endif
-
-  if(cache->name) {
-    /* draw cache name */
-    cairo_text_extents (cr, cache->name, &extents);
-    y += extents.height;
-    cairo_move_to (cr, x, y);
-    cairo_set_source_rgba (cr, 0, 0, 0, 1);
-    cairo_show_text (cr, cache->name);
-    cairo_stroke (cr);
-  
-    /* return to the left border and below text */
-    y += LINE_SKIP;
-    x = rect->x;
-  }
-
-  if(cache->terrain) {
-    /* draw cache rating */
-    const char *terrain = "Terrain:"; 
-    icon = icon_get(ICON_STARS, (int)(cache->terrain*2-2));
-    cairo_text_extents (cr, _(terrain), &extents);
-    y += (gdk_pixbuf_get_height(icon) + extents.height)/2;
-
-    /* draw "Terrain:" string */
-    cairo_move_to (cr, x, y);
-    cairo_set_source_rgba (cr, 0, 0, 0, 1);
-    cairo_show_text (cr, _(terrain));
-    cairo_stroke (cr);
-    x += extents.width + 2;
-  
-    /* draw terrain stars */
-    cairo_draw_pixbuf(cr, icon, x, y - 
-		      (gdk_pixbuf_get_height(icon) + extents.height)/2);
-
-    x += gdk_pixbuf_get_width(icon) + LINE_SKIP;
+    
+    if(cache->name) {
+      /* draw cache name */
+      cairo_text_extents (event->data.draw.cr, cache->name, &extents);
+      y += extents.height;
+      cairo_move_to (event->data.draw.cr, x, y);
+      cairo_set_source_rgba (event->data.draw.cr, 0, 0, 0, 1);
+      cairo_show_text (event->data.draw.cr, cache->name);
+      cairo_stroke (event->data.draw.cr);
+      
+      /* return to the left border and below text */
+      y += LINE_SKIP;
+      x = event->data.draw.rect->x;
+    }
+    
+    if(cache->terrain) {
+      /* draw cache rating */
+      const char *terrain = "Terrain:"; 
+      icon = icon_get(ICON_STARS, (int)(cache->terrain*2-2));
+      cairo_text_extents (event->data.draw.cr, _(terrain), &extents);
+      y += (gdk_pixbuf_get_height(icon) + extents.height)/2;
+      
+      /* draw "Terrain:" string */
+      cairo_move_to (event->data.draw.cr, x, y);
+      cairo_set_source_rgba (event->data.draw.cr, 0, 0, 0, 1);
+      cairo_show_text (event->data.draw.cr, _(terrain));
+      cairo_stroke (event->data.draw.cr);
+      x += extents.width + 2;
+      
+      /* draw terrain stars */
+      cairo_draw_pixbuf(event->data.draw.cr, icon, x, y - 
+			(gdk_pixbuf_get_height(icon) + extents.height)/2);
+      
+      x += gdk_pixbuf_get_width(icon) + LINE_SKIP;
     y -= (gdk_pixbuf_get_height(icon) + extents.height)/2;
-  }
-
-  if(cache->difficulty) {
-    const char *difficulty = "Difficulty:"; 
-    cairo_text_extents (cr, _(difficulty), &extents);
-    y += (gdk_pixbuf_get_height(icon) + extents.height)/2;
+    }
     
-    /* draw "Difficulty:" string */
-    cairo_move_to (cr, x, y);
-    cairo_set_source_rgba (cr, 0, 0, 0, 1);
-    cairo_show_text (cr, _(difficulty));
-    cairo_stroke (cr);
-    x += extents.width + 2;
-    
-    icon = icon_get(ICON_STARS, (int)(cache->difficulty*2-2));
-    cairo_draw_pixbuf(cr, icon, x, y - 
+    if(cache->difficulty) {
+      const char *difficulty = "Difficulty:"; 
+      cairo_text_extents (event->data.draw.cr, _(difficulty), &extents);
+      y += (gdk_pixbuf_get_height(icon) + extents.height)/2;
+      
+      /* draw "Difficulty:" string */
+      cairo_move_to (event->data.draw.cr, x, y);
+      cairo_set_source_rgba (event->data.draw.cr, 0, 0, 0, 1);
+      cairo_show_text (event->data.draw.cr, _(difficulty));
+      cairo_stroke (event->data.draw.cr);
+      x += extents.width + 2;
+      
+      icon = icon_get(ICON_STARS, (int)(cache->difficulty*2-2));
+      cairo_draw_pixbuf(event->data.draw.cr, icon, x, y - 
 		      (gdk_pixbuf_get_height(icon) + extents.height)/2);
+    }
+  } else if(event->type == OSM_GPS_MAP_BALLOON_EVENT_TYPE_CLICK) {
+    printf("click %s event at %d %d\n", 
+	   event->data.click.down?"down":"up",
+	   event->data.click.x, event->data.click.y);
+
+    /* make the main screen jump to that cache */
+    if(!event->data.click.down) {
+      if(context->appdata->cur_cache) {
+	printf("ERROR: no current cache should be visible!\n");
+      } else {
+	gpx_t *is_in = NULL;
+
+	if(!context->appdata->cur_gpx) {
+	  printf("click while in \"all\" view\n");
+
+	  /* we first need to figure out which gpx file this cache */
+	  /* is in so we can open it first */
+	  gpx_t *gpx = context->appdata->gpx;
+	  while(gpx && !is_in) {
+	    cache_t *cur = gpx->cache;
+	    while(cur && !is_in) {
+	      if(cur == cache)
+		is_in = gpx;
+	      cur = cur->next;
+	    }
+	    gpx = gpx->next;
+	  }
+	  
+	  if(is_in)
+	    gpxlist_goto_cachelist(context->appdata, is_in);
+
+	} else 
+	  /* the simple case: there already is an open gpx file and */
+	  /* we just jump into the "cache" view */
+	  is_in = context->appdata->cur_gpx;
+
+	if(is_in) {
+	  printf("selecting %s in %s\n", 
+		 cache->id,
+		 context->appdata->cur_gpx->name);
+
+	  cachelist_goto_cache(context->appdata, cache);
+
+	  /* give focus to main screen (important for maemo) */
+	  printf("raising main window\n");
+	  gtk_window_present(GTK_WINDOW(context->appdata->window));
+	}
+      }
+    }
+  } else if(event->type == OSM_GPS_MAP_BALLOON_EVENT_TYPE_REMOVED) {
+    printf("removed\n");
+    context->balloon = NULL;
   }
 }
 
@@ -456,8 +629,9 @@ on_map_button_release_event(GtkWidget *widget,
       float dist = gpx_pos_get_distance(pos, cpos, FALSE);
       if(dist2pixel(context, dist, cpos.lat) < CLICK_FUZZ) {
 	
+	context->balloon = nearest;
 	osm_gps_map_osd_draw_balloon(map, cpos.lat, cpos.lon,
-				     balloon_draw_cb, nearest);
+				     balloon_cb, context);
       }
     }
     context->press_on = NULL;
@@ -475,8 +649,6 @@ on_map_button_release_event(GtkWidget *widget,
 static void on_window_destroy(GtkWidget *widget, map_context_t *context) {
   appdata_t *appdata = context->appdata;
   
-  printf("destroy map window\n");
-
   /* save map parameters */
   OsmGpsMap *map = OSM_GPS_MAP(context->widget);
   gint zoom;
@@ -498,6 +670,7 @@ static void on_window_destroy(GtkWidget *widget, map_context_t *context) {
 #endif
 
   gtk_timeout_remove(context->handler_id);
+  gps_unregister_callback(appdata, context->cb_id);
 
   g_free(context);
   appdata->map.context = NULL;
@@ -522,80 +695,10 @@ on_window_realize(GtkWidget *widget, gpointer data) {
 }
 #endif
 
-/* draw geocaches and set window title */
-static void map_setup(map_context_t *context) {
-  char *name = NULL;
+/* on maemo a window is either on top or completely invisible. this */
+/* means that we only need to update the map window if its raised.  */
+/* on ordinary desktops this is different and we always update */
 
-  if(!context->appdata->cur_gpx && !context->appdata->cur_cache) {
-    if(context->state != MAP_ALL) {
-      printf("map_setp(ALL)\n");
-
-      /* no navigation in this mode */
-      osm_gps_map_osd_clear_nav (OSM_GPS_MAP(context->widget));
-
-      /* clear all existing ccahe images */
-      osm_gps_map_clear_images (OSM_GPS_MAP(context->widget));
-
-      /* draw all geocaches */
-      gpx_t *gpx = context->appdata->gpx;
-      while(gpx) {
-	map_draw_gpx(context->appdata, context->widget, gpx);
-	gpx = gpx->next;
-      }
-      name = g_strdup(_("all"));
-      context->state = MAP_ALL;
-    }
-  } else if(!context->appdata->cur_cache) {
-    if(context->state != MAP_GPX) {
-      printf("map_setp(GPX)\n");
-
-      /* no navigation in this mode */
-      osm_gps_map_osd_clear_nav (OSM_GPS_MAP(context->widget));
-
-      /* clear all existing ccahe images */
-      osm_gps_map_clear_images (OSM_GPS_MAP(context->widget));
-
-      map_draw_gpx(context->appdata, context->widget, 
-		   context->appdata->cur_gpx);
-      name = g_strdup(context->appdata->cur_gpx->name);
-      context->state = MAP_GPX;
-    }
-  } else {
-    cache_t *cache = context->appdata->cur_cache;
-    
-    printf("map_setp(CACHE)\n");
-    
-    /* no balloons in this mode */
-    osm_gps_map_osd_clear_balloon (OSM_GPS_MAP(context->widget));
-    
-    /* clear all existing ccahe images */
-    osm_gps_map_clear_images (OSM_GPS_MAP(context->widget));
-    
-    map_draw_cache(context->widget, cache);
-    name = g_strdup(cache->name);
-    context->state = MAP_CACHE;
-    
-    /* navigation in this mode! */
-    pos_t cpos = gpx_cache_pos(cache);
-    osm_gps_map_osd_draw_nav (OSM_GPS_MAP(context->widget), 
-			      context->appdata->imperial,
-			      cpos.lat, cpos.lon, cache->name);
-  }
-
-  if(name) {
-    char *title = g_strdup_printf(_("Map - %s"), name);
-    g_free(name);
-
-    gtk_window_set_title(GTK_WINDOW(context->window), title);
-    
-    g_free(title);
-  } else
-    printf("map_setup(keep)\n");
-}
-
-/* on maemo a window is either on top or completely invisible. this means that */
-/* we only need to update the map window if its raised. on ordinary desktops this */
-/* is different and we always update */
 static gboolean on_focus_in(GtkWidget *widget, GdkEventFocus *event,
 			 gpointer data) {
   printf("map got focus\n");
@@ -604,12 +707,19 @@ static gboolean on_focus_in(GtkWidget *widget, GdkEventFocus *event,
 }  
 
 void map_update(appdata_t *appdata) {
+  printf("map_update\n");
 #ifndef USE_MAEMO
   if(appdata->map.context) 
     map_setup(appdata->map.context);
 #endif
 }
 
+/* callback called by the gps layer whenever gps state changes */
+static void 
+gps_callback(struct gps_state *state, gpointer data) {
+  printf("map: gps callback\n");
+}
+
 void map(appdata_t *appdata) {
   map_context_t *context = NULL; 
 
@@ -682,9 +792,6 @@ void map(appdata_t *appdata) {
   context->window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
 #endif
 
-  /* setup cache state */
-  map_setup(context);
-
 #ifndef USE_MAEMO
   gtk_window_set_default_size(GTK_WINDOW(context->window), 640, 480);
 #endif
@@ -703,6 +810,7 @@ void map(appdata_t *appdata) {
 
   /* install handler for timed updates of the gps button */
   context->handler_id = gtk_timeout_add(1000, map_gps_update, context);
+  context->cb_id = gps_register_callback(appdata, gps_callback, context);
 
 #if MAEMO_VERSION_MAJOR == 5
   /* prevent some of the main screen things */
