@@ -27,13 +27,18 @@ typedef struct map_context_s {
   GtkWidget *window;
   GtkWidget *widget;
   gint handler_id;
-  cache_t *press_on;
+  cache_t *balloon, *press_on;
   gboolean map_complete;
   map_state_t state;
+  void *cb_id;
 
 #if MAEMO_VERSION_MAJOR == 5
   GtkWidget *old_view;
 #endif
+
+  /* save various values defining the overlay state */
+  /* in order to repaint the overlays only if necessary */
+
 } map_context_t;
 
 
