@@ -475,3 +475,27 @@ GtkWidget *link_button_by_id(appdata_t *appdata, char *str,
   return gtk_label_new(str);
 }
 
+
+GtkWidget *link_icon_button_by_id(appdata_t *appdata, GtkWidget *icon, 
+			     const char *type, int id) {
+
+#ifdef ENABLE_BROWSER_INTERFACE
+  if(id) {
+    GtkWidget *ref = gtk_button_new();
+    gtk_button_set_image(GTK_BUTTON(ref), icon);
+
+#if defined(USE_MAEMO) && (MAEMO_VERSION_MAJOR == 5)
+    //    hildon_gtk_widget_set_theme_size(ref, 
+    //	       (HILDON_SIZE_FINGER_HEIGHT | HILDON_SIZE_AUTO_WIDTH));
+#endif  
+    g_object_set_data(G_OBJECT(ref), "id", (gpointer)id);
+    g_object_set_data(G_OBJECT(ref), "type", (gpointer)type);
+    gtk_signal_connect(GTK_OBJECT(ref), "clicked",
+		       GTK_SIGNAL_FUNC(on_link_id_clicked), appdata);
+    
+    return ref;
+  }
+#endif
+  return icon;
+}
+
