@@ -135,6 +135,13 @@ typedef struct {
 
 #define CLOSE_BUTTON_RADIUS   (BALLOON_CORNER_RADIUS)
 
+#if 0
+#define FIN   printf("entering function %s\n", __func__);
+#define FOUT  printf("leaving function %s\n", __func__);
+#else
+#define FIN   ;
+#define FOUT  ;
+#endif
 
 /* draw the bubble shape. this is used twice, once for the shape and once */
 /* for the shadow */
@@ -142,6 +149,8 @@ static void
 osm_gps_map_draw_balloon_shape (cairo_t *cr, int x0, int y0, int x1, int y1, 
        gboolean bottom, int px, int py, int px0, int px1) {
 
+    FIN;
+
     cairo_move_to (cr, x0, y0 + BALLOON_CORNER_RADIUS);
     cairo_arc (cr, x0 + BALLOON_CORNER_RADIUS, y0 + BALLOON_CORNER_RADIUS,
                BALLOON_CORNER_RADIUS, -M_PI, -M_PI/2);
@@ -170,12 +179,19 @@ osm_gps_map_draw_balloon_shape (cairo_t *cr, int x0, int y0, int x1, int y1,
                BALLOON_CORNER_RADIUS, M_PI/2, M_PI);
 
     cairo_close_path (cr);
+
+    FOUT;
 }
 
 static void
 osd_render_balloon(osm_gps_map_osd_t *osd) {
+    FIN;
+
     osd_priv_t *priv = (osd_priv_t*)osd->priv; 
 
+    if(!priv->balloon.surface)
+        return;
+
     /* get zoom */
     gint zoom;
     g_object_get(OSM_GPS_MAP(osd->widget), "zoom", &zoom, NULL);
@@ -266,25 +282,35 @@ osd_render_balloon(osm_gps_map_osd_t *osd) {
     cairo_stroke (cr);
     
     if (priv->balloon.cb) {
+        osm_gps_map_balloon_event_t event;
+
         /* clip in case application tries to draw in */
             /* exceed of the balloon */
         cairo_rectangle (cr, priv->balloon.rect.x, priv->balloon.rect.y, 
                          priv->balloon.rect.w, priv->balloon.rect.h);
         cairo_clip (cr);
-        cairo_new_path (cr);  /* current path is not
-                                 consumed by cairo_clip() */
+        cairo_new_path (cr);  /* current path is not consumed by cairo_clip */
+
+        /* request the application to draw the balloon contents */
+        event.type = OSM_GPS_MAP_BALLOON_EVENT_TYPE_DRAW;
+        event.data.draw.rect = &priv->balloon.rect;
+        event.data.draw.cr = cr;
         
-        priv->balloon.cb(cr, &priv->balloon.rect, priv->balloon.data);
+        priv->balloon.cb(&event, priv->balloon.data);
     }
 
     cairo_destroy(cr);
+
+    FOUT;
 }
 
 /* return true if balloon is being displayed and if */
 /* the given coordinate is within this balloon */
 static gboolean 
-osd_balloon_check(osm_gps_map_osd_t *osd, gboolean down, gint x, gint y) 
+osd_balloon_check(osm_gps_map_osd_t *osd, gboolean click, gboolean down, gint x, gint y) 
 {
+    FIN;
+
     osd_priv_t *priv = (osd_priv_t*)osd->priv; 
 
     if(!priv->balloon.surface)
@@ -302,19 +328,45 @@ osd_balloon_check(osm_gps_map_osd_t *osd, gboolean down, gint x, gint y)
         (x > xs) && (x < xs + priv->balloon.rect.w) &&
         (y > ys) && (y < ys + priv->balloon.rect.h);
     
-    /* handle the fact that the balloon may have been created by the */
-    /* button down event */
-    if(!is_in && !down && !priv->balloon.just_created) {
-        /* the user actually clicked outside the balloon */
+    /* is this a real click or is the application just checking for something? */
+    if(click) {
 
-        /* close the balloon! */
-        osm_gps_map_osd_clear_balloon (OSM_GPS_MAP(osd->widget));
+        /* handle the fact that the balloon may have been created by the */
+        /* button down event */
+        if(!is_in && !down && !priv->balloon.just_created) {
+            /* the user actually clicked outside the balloon */
+            
+            /* close the balloon! */
+            osm_gps_map_osd_clear_balloon (OSM_GPS_MAP(osd->widget));
+
+            /* and inform application about this */
+            if(priv->balloon.cb) {
+                osm_gps_map_balloon_event_t event;
+                event.type = OSM_GPS_MAP_BALLOON_EVENT_TYPE_REMOVED;
+                priv->balloon.cb(&event, priv->balloon.data);
+            }
+
+        }
+        
+        if(is_in && priv->balloon.cb) {
+            osm_gps_map_balloon_event_t event;
+            
+            /* notify application of click */
+            event.type = OSM_GPS_MAP_BALLOON_EVENT_TYPE_CLICK;
+            event.data.click.x = x - xs;
+            event.data.click.y = y - ys;
+            event.data.click.down = down;
+            
+            priv->balloon.cb(&event, priv->balloon.data);
+        }
     }
-
+    FOUT;
     return is_in;
 }
 
 void osm_gps_map_osd_clear_balloon (OsmGpsMap *map) {
+    FIN;
+
     g_return_if_fail (OSM_IS_GPS_MAP (map));
 
     osm_gps_map_osd_t *osd = osm_gps_map_osd_get(map);
@@ -330,11 +382,13 @@ void osm_gps_map_osd_clear_balloon (OsmGpsMap *map) {
         priv->balloon.lon = OSM_GPS_MAP_INVALID;
     }
     osm_gps_map_redraw(map);
+    FOUT;
 }
 
 void 
 osm_gps_map_osd_draw_balloon (OsmGpsMap *map, float latitude, float longitude, 
                               OsmGpsMapBalloonCallback cb, gpointer data) {
+    FIN;
     g_return_if_fail (OSM_IS_GPS_MAP (map));
 
     osm_gps_map_osd_t *osd = osm_gps_map_osd_get(map);
@@ -361,6 +415,7 @@ osm_gps_map_osd_draw_balloon (OsmGpsMap *map, float latitude, float longitude,
     osd_render_balloon(osd);
 
     osm_gps_map_redraw(map);
+    FOUT;
 }
 
 #endif // OSD_BALLOON
@@ -505,6 +560,7 @@ osd_shape(cairo_t *cr) {
 static gboolean
 osm_gps_map_in_circle(gint x, gint y, gint cx, gint cy, gint rad) 
 {
+    FIN;
     return( pow(cx - x, 2) + pow(cy - y, 2) < rad * rad);
 }
 
@@ -595,7 +651,7 @@ osd_check_zoom(gint x, gint y) {
 /* internal value to draw the arrow on the "puller" */
 #define OSD_S_D0  (OSD_S_RAD/2)
 #ifndef OSD_FONT_SIZE
-#define OSD_FONT_SIZE 16.0
+#define OSD_FONT_SIZE (16.0)
 #endif
 #define OSD_TEXT_BORDER   (OSD_FONT_SIZE/2)
 #define OSD_TEXT_SKIP     (OSD_FONT_SIZE/8)
@@ -703,9 +759,12 @@ osd_source_content(osm_gps_map_osd_t *osd, cairo_t *cr, gint offset) {
 
 static void
 osd_render_source_sel(osm_gps_map_osd_t *osd, gboolean force_rerender) {
+    FIN;
+
     osd_priv_t *priv = (osd_priv_t*)osd->priv; 
 
-    if(priv->source_sel.rendered && !force_rerender)
+    if(!priv->source_sel.surface ||
+       (priv->source_sel.rendered && !force_rerender))
         return;
 
     priv->source_sel.rendered = TRUE;
@@ -749,12 +808,14 @@ osd_render_source_sel(osm_gps_map_osd_t *osd, gboolean force_rerender) {
     cairo_stroke (cr);
 
     cairo_destroy(cr);
+    FOUT;
 }
 
 /* re-allocate the buffer used to draw the menu. This is used */
 /* to collapse/expand the buffer */
 static void
 osd_source_reallocate(osm_gps_map_osd_t *osd) {
+    FIN;
     osd_priv_t *priv = (osd_priv_t*)osd->priv; 
 
     /* re-allocate offscreen bitmap */
@@ -795,12 +856,14 @@ osd_source_reallocate(osm_gps_map_osd_t *osd) {
         cairo_image_surface_create(CAIRO_FORMAT_ARGB32, w+2, h+2);
 
     osd_render_source_sel(osd, TRUE);
+    FOUT;
 }
 
 #define OSD_HZ      15
 #define OSD_TIME    500
 
 static gboolean osd_source_animate(gpointer data) {
+    FIN;
     osm_gps_map_osd_t *osd = (osm_gps_map_osd_t*)data;
     osd_priv_t *priv = (osd_priv_t*)osd->priv; 
     int diff = OSD_S_EXP_W - OSD_S_W - OSD_S_X;
@@ -839,6 +902,7 @@ static gboolean osd_source_animate(gpointer data) {
     if(done) 
         priv->source_sel.handler_id = 0;
 
+    FOUT;
     return !done;
 }
 
@@ -846,6 +910,7 @@ static gboolean osd_source_animate(gpointer data) {
 static void
 osd_source_toggle(osm_gps_map_osd_t *osd)
 {
+    FIN;
     osd_priv_t *priv = (osd_priv_t*)osd->priv; 
 
     /* ignore clicks while animation is running */
@@ -871,11 +936,13 @@ osd_source_toggle(osm_gps_map_osd_t *osd)
     /* start timer to handle animation */
     priv->source_sel.handler_id = gtk_timeout_add(OSD_TIME/OSD_HZ, 
                                                   osd_source_animate, osd);
+    FOUT;
 }
 
 /* check if the user clicked inside the source selection area */
 static osd_button_t
 osd_source_check(osm_gps_map_osd_t *osd, gboolean down, gint x, gint y) {
+    FIN;
     osd_priv_t *priv = (osd_priv_t*)osd->priv; 
 
     if(!priv->source_sel.expanded)
@@ -933,19 +1000,21 @@ osd_source_check(osm_gps_map_osd_t *osd, gboolean down, gint x, gint y) {
                     osm_gps_map_repaint(OSM_GPS_MAP(osd->widget));
                 }
             }
-
+            
             /* return "clicked in OSD background" to prevent further */
             /* processing by application */
             return OSD_BG;
         }
     }
 
+    FOUT;
     return OSD_NONE;
 }
 #endif // OSD_SOURCE_SEL
 
 static osd_button_t
-osd_check(osm_gps_map_osd_t *osd, gboolean down, gint x, gint y) {
+osd_check_int(osm_gps_map_osd_t *osd, gboolean click, gboolean down, gint x, gint y) {
+    FIN;
     osd_button_t but = OSD_NONE;
 
 #ifdef OSD_BALLOON
@@ -986,11 +1055,12 @@ osd_check(osm_gps_map_osd_t *osd, gboolean down, gint x, gint y) {
 #ifdef OSD_BALLOON
     if(but == OSD_NONE) {
         /* check if user clicked into balloon */
-        if(osd_balloon_check(osd, down, x, y)) 
+        if(osd_balloon_check(osd, click, down, x, y)) 
             but = OSD_BG;
     }
 #endif
         
+    FOUT;
     return but;
 }
 
@@ -1067,7 +1137,7 @@ osd_zoom_labels(cairo_t *cr, gint x, gint y) {
 #ifdef OSD_COORDINATES
 
 #ifndef OSD_COORDINATES_FONT_SIZE
-#define OSD_COORDINATES_FONT_SIZE 12
+#define OSD_COORDINATES_FONT_SIZE (12.0)
 #endif
 
 #define OSD_COORDINATES_OFFSET (OSD_COORDINATES_FONT_SIZE/6)
@@ -1134,15 +1204,24 @@ static char
 /* render a string at the given screen position */
 static int
 osd_render_centered_text(cairo_t *cr, int y, int width, char *text) {
-    char *p = g_strdup(text);
+    FIN;
+
+    if(!text) return y;
+
+    char *p = g_malloc(strlen(text)+4);  // space for "...\n"
+    strcpy(p, text);
+
     cairo_text_extents_t extents;
     cairo_text_extents (cr, p, &extents);
+    g_assert(extents.width != 0.0);
 
     /* check if text needs to be truncated */
-    int len = strlen(text)-2;
+    int trunc_at = strlen(text)-1;
     while(extents.width > width) {
-        len--;
-        strcpy(p+len, "...");
+        g_assert(trunc_at > 0);
+
+        trunc_at--;
+        strcpy(p+trunc_at, "...");
         cairo_text_extents (cr, p, &extents);
     }
 
@@ -1158,15 +1237,20 @@ osd_render_centered_text(cairo_t *cr, int y, int width, char *text) {
 
     g_free(p);
 
-    /* skip + 1/4 line */
-    return y + 5*OSD_COORDINATES_FONT_SIZE/4;
+    /* skip + 1/5 line */
+    FOUT;
+    return y + 6*OSD_COORDINATES_FONT_SIZE/5;
 }
 
 static void
 osd_render_coordinates(osm_gps_map_osd_t *osd) 
 {
+    FIN;
     osd_priv_t *priv = (osd_priv_t*)osd->priv; 
 
+    if(!priv->coordinates.surface)
+        return;
+
     /* get current map position */
     gfloat lat, lon;
     g_object_get(osd->widget, "latitude", &lat, "longitude", &lon, NULL);
@@ -1182,6 +1266,7 @@ osd_render_coordinates(osm_gps_map_osd_t *osd)
     priv->coordinates.lon = lon;
 
     /* first fill with transparency */
+
     cairo_t *cr = cairo_create(priv->coordinates.surface);
     cairo_set_operator(cr, CAIRO_OPERATOR_SOURCE);
     //    cairo_set_source_rgba(cr, 1.0, 1.0, 1.0, 0.5);
@@ -1205,6 +1290,7 @@ osd_render_coordinates(osm_gps_map_osd_t *osd)
     g_free(longitude);
 
     cairo_destroy(cr);
+    FOUT;
 }
 #endif  // OSD_COORDINATES
 
@@ -1232,6 +1318,7 @@ static float get_distance(float lat1, float lon1, float lat2, float lon2) {
 static void
 osd_render_nav(osm_gps_map_osd_t *osd) 
 {
+    FIN;
     osd_priv_t *priv = (osd_priv_t*)osd->priv; 
 
     if(!priv->nav.surface || isnan(priv->nav.lat) || isnan(priv->nav.lon))
@@ -1336,9 +1423,11 @@ osd_render_nav(osm_gps_map_osd_t *osd)
     }
 
     cairo_destroy(cr);
+    FOUT;
 }
 
 void osm_gps_map_osd_clear_nav (OsmGpsMap *map) {
+    FIN;
     g_return_if_fail (OSM_IS_GPS_MAP (map));
 
     osm_gps_map_osd_t *osd = osm_gps_map_osd_get(map);
@@ -1355,11 +1444,13 @@ void osm_gps_map_osd_clear_nav (OsmGpsMap *map) {
         if(priv->nav.name) g_free(priv->nav.name);
     }
     osm_gps_map_redraw(map);
+    FOUT;
 }
 
 void 
 osm_gps_map_osd_draw_nav (OsmGpsMap *map, gboolean imperial,
                           float latitude, float longitude, char *name) {
+    FIN;
     g_return_if_fail (OSM_IS_GPS_MAP (map));
 
     osm_gps_map_osd_t *osd = osm_gps_map_osd_get(map);
@@ -1383,6 +1474,7 @@ osm_gps_map_osd_draw_nav (OsmGpsMap *map, gboolean imperial,
     osd_render_nav(osd);
 
     osm_gps_map_redraw(map);
+    FOUT;
 }
 
 #endif // OSD_NAV
@@ -1401,6 +1493,7 @@ osm_gps_map_osd_draw_nav (OsmGpsMap *map, gboolean imperial,
 
 static void
 osd_render_crosshair_shape(cairo_t *cr) {
+    FIN;
     cairo_arc (cr, OSD_CROSSHAIR_W/2, OSD_CROSSHAIR_H/2, 
                OSD_CROSSHAIR_RADIUS, 0,  2*M_PI);
 
@@ -1419,14 +1512,16 @@ osd_render_crosshair_shape(cairo_t *cr) {
     cairo_rel_line_to (cr, 0, OSD_CROSSHAIR_TICK);
 
     cairo_stroke (cr);
+    FOUT;
 }
 
 static void
 osd_render_crosshair(osm_gps_map_osd_t *osd) 
 {
+    FIN;
     osd_priv_t *priv = (osd_priv_t*)osd->priv; 
 
-    if(priv->crosshair.rendered)
+    if(!priv->crosshair.surface || priv->crosshair.rendered)
         return;
 
     priv->crosshair.rendered = TRUE;
@@ -1449,13 +1544,14 @@ osd_render_crosshair(osm_gps_map_osd_t *osd)
     osd_render_crosshair_shape(cr);
 
     cairo_destroy(cr);
+    FOUT;
 }
 #endif
 
 #ifdef OSD_SCALE
 
 #ifndef OSD_SCALE_FONT_SIZE
-#define OSD_SCALE_FONT_SIZE 12
+#define OSD_SCALE_FONT_SIZE (12.0)
 #endif
 #define OSD_SCALE_W   (10*OSD_SCALE_FONT_SIZE)
 #define OSD_SCALE_H   (5*OSD_SCALE_FONT_SIZE/2)
@@ -1470,8 +1566,12 @@ osd_render_crosshair(osm_gps_map_osd_t *osd)
 static void
 osd_render_scale(osm_gps_map_osd_t *osd) 
 {
+    FIN;
     osd_priv_t *priv = (osd_priv_t*)osd->priv; 
 
+    if(!priv->scale.surface)
+        return;
+
     /* this only needs to be rendered if the zoom has changed */
     gint zoom;
     g_object_get(OSM_GPS_MAP(osd->widget), "zoom", &zoom, NULL);
@@ -1593,14 +1693,19 @@ osd_render_scale(osm_gps_map_osd_t *osd)
     cairo_stroke(cr);
 
     cairo_destroy(cr);
+    FOUT;
 }
 #endif
 
 static void
 osd_render_controls(osm_gps_map_osd_t *osd) 
 {
+    FIN;
     osd_priv_t *priv = (osd_priv_t*)osd->priv; 
 
+    if(!priv->controls.surface)
+        return;
+
     if(priv->controls.rendered 
 #ifdef OSD_GPS_BUTTON
        && (priv->controls.gps_enabled == (osd->cb != NULL))
@@ -1872,11 +1977,13 @@ osd_draw(osm_gps_map_osd_t *osd, GdkDrawable *drawable)
 #endif
 
     cairo_destroy(cr);
+    FOUT;
 }
 
 static void
 osd_free(osm_gps_map_osd_t *osd) 
 {
+    FIN;
     osd_priv_t *priv = (osd_priv_t *)(osd->priv);
 
     if (priv->controls.surface)
@@ -1916,11 +2023,13 @@ osd_free(osm_gps_map_osd_t *osd)
 #endif
 
     g_free(priv);
+    FOUT;
 }
 
 static gboolean
 osd_busy(osm_gps_map_osd_t *osd) 
 {
+    FIN;
 #ifdef OSD_SOURCE_SEL
     osd_priv_t *priv = (osd_priv_t *)(osd->priv);
     return (priv->source_sel.handler_id != 0);
@@ -1929,6 +2038,12 @@ osd_busy(osm_gps_map_osd_t *osd)
 #endif
 }
 
+static osd_button_t
+osd_check(osm_gps_map_osd_t *osd, gboolean down, gint x, gint y) {
+    FIN;
+    return osd_check_int(osd, TRUE, down, x, y);
+}
+
 static osm_gps_map_osd_t osd_classic = {
     .widget     = NULL,
 
@@ -1948,6 +2063,7 @@ static osm_gps_map_osd_t osd_classic = {
 void
 osm_gps_map_osd_classic_init(OsmGpsMap *map) 
 {
+    FIN;
     osd_priv_t *priv = osd_classic.priv = g_new0(osd_priv_t, 1);
 
 #ifdef OSD_BALLOON
@@ -1958,6 +2074,7 @@ osm_gps_map_osd_classic_init(OsmGpsMap *map)
     osd_classic.priv = priv;
 
     osm_gps_map_register_osd(map, &osd_classic);
+    FOUT;
 }
 
 #ifdef OSD_GPS_BUTTON
@@ -1965,6 +2082,7 @@ osm_gps_map_osd_classic_init(OsmGpsMap *map)
 /* but instead are to be used by the main application */
 void osm_gps_map_osd_enable_gps (OsmGpsMap *map, OsmGpsMapOsdCallback cb, 
                                  gpointer data) {
+    FIN;
     osm_gps_map_osd_t *osd = osm_gps_map_osd_get(map);
     g_return_if_fail (osd);
 
@@ -1976,13 +2094,15 @@ void osm_gps_map_osd_enable_gps (OsmGpsMap *map, OsmGpsMapOsdCallback cb,
     osd->render(osd);
 
     osm_gps_map_redraw(map);
+    FOUT;
 }
 #endif
 
 osd_button_t
 osm_gps_map_osd_check(OsmGpsMap *map, gint x, gint y) {
+    FIN;
     osm_gps_map_osd_t *osd = osm_gps_map_osd_get(map);
     g_return_val_if_fail (osd, OSD_NONE);
     
-    return osd_check(osd, TRUE, x, y);
+    return osd_check_int(osd, FALSE, TRUE, x, y);
 }
