@@ -381,7 +381,12 @@ view_selection_func(GtkTreeSelection *selection, GtkTreeModel *model,
 static GtkWidget *location_widget(location_context_t *context) {
 
   GtkWidget *vbox = gtk_vbox_new(FALSE,3);
+
+#ifndef USE_PANNABLE_AREA
   context->view = gtk_tree_view_new();
+#else
+  context->view = hildon_gtk_tree_view_new(HILDON_UI_MODE_EDIT);
+#endif
 
   gtk_tree_selection_set_select_function(
 	 gtk_tree_view_get_selection(GTK_TREE_VIEW(context->view)), 
@@ -451,46 +456,57 @@ static GtkWidget *location_widget(location_context_t *context) {
   
   g_object_unref(context->store);
 
-#if 0
-  /* make list react on clicks */
-  g_signal_connect(context->view, "row-activated", 
-		   (GCallback)gpxlist_view_onRowActivated, appdata);
-#endif
-
   /* select the "active" row */
   location_select(context);
 
   /* put it into a scrolled window */
+#ifndef USE_PANNABLE_AREA
   GtkWidget *scrolled_window = gtk_scrolled_window_new (NULL, NULL);
   gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled_window), 
 				 GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
   gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(scrolled_window), 
 				      GTK_SHADOW_ETCHED_IN);
-  //  gtk_container_set_border_width(GTK_CONTAINER(scrolled_window), 3);
   gtk_container_add(GTK_CONTAINER(scrolled_window), context->view);
   gtk_box_pack_start_defaults(GTK_BOX(vbox), scrolled_window);
+#else
+  GtkWidget *pannable_area = hildon_pannable_area_new();
+  gtk_container_add(GTK_CONTAINER(pannable_area), context->view);
+  gtk_box_pack_start_defaults(GTK_BOX(vbox), pannable_area);
+#endif
 
   /* ------- button box ------------ */
 
   GtkWidget *hbox = gtk_hbox_new(TRUE,3);
   context->but_add = gtk_button_new_with_label(_("Add"));
+#if defined(USE_MAEMO) && (MAEMO_VERSION_MAJOR >= 5)
+  hildon_gtk_widget_set_theme_size(context->but_add, 
+	     (HILDON_SIZE_FINGER_HEIGHT | HILDON_SIZE_AUTO_WIDTH));
+#endif  
   gtk_box_pack_start_defaults(GTK_BOX(hbox), context->but_add);
   gtk_signal_connect(GTK_OBJECT(context->but_add), "clicked",
 		     GTK_SIGNAL_FUNC(on_location_add), context);
 
   context->but_edit = gtk_button_new_with_label(_("Edit"));
+#if defined(USE_MAEMO) && (MAEMO_VERSION_MAJOR >= 5)
+  hildon_gtk_widget_set_theme_size(context->but_edit, 
+	     (HILDON_SIZE_FINGER_HEIGHT | HILDON_SIZE_AUTO_WIDTH));
+#endif  
   gtk_box_pack_start_defaults(GTK_BOX(hbox), context->but_edit);
   gtk_signal_connect(GTK_OBJECT(context->but_edit), "clicked",
 		     GTK_SIGNAL_FUNC(on_location_edit), context);
 
   context->but_remove = gtk_button_new_with_label(_("Remove"));
+#if defined(USE_MAEMO) && (MAEMO_VERSION_MAJOR >= 5)
+  hildon_gtk_widget_set_theme_size(context->but_remove, 
+	     (HILDON_SIZE_FINGER_HEIGHT | HILDON_SIZE_AUTO_WIDTH));
+#endif  
   gtk_widget_set_sensitive(context->but_remove, 
 			   context->appdata->active_location);
   gtk_box_pack_start_defaults(GTK_BOX(hbox), context->but_remove);
   gtk_signal_connect(GTK_OBJECT(context->but_remove), "clicked",
       GTK_SIGNAL_FUNC(on_location_remove), context);
 
-  gtk_box_pack_start_defaults(GTK_BOX(vbox), hbox);
+  gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
   return vbox;
 }
 
@@ -509,9 +525,12 @@ void cb_menu_settings(GtkWidget *window, gpointer data) {
 #if defined(USE_MAEMO) && defined(HILDON_HELP)
   hildon_help_dialog_help_enable(GTK_DIALOG(dialog), 
 		 HELP_ID_SETTINGS, appdata->osso_context);
-  gtk_window_set_default_size(GTK_WINDOW(dialog), 550, 100);
 #endif 
 
+#if defined(USE_MAEMO) && (MAEMO_VERSION_MAJOR >= 5)
+  gtk_window_set_default_size(GTK_WINDOW(dialog), 550, 300);
+#endif
+
   gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox),
 		     notebook = gtk_notebook_new(), TRUE, TRUE, 0);
 
